 /*
 *Driver for the card pcie bridge.

 *Copyright (C) 2019 AVerMedia TECHNOLOGIES, Inc.

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
#include "cxt_mgr.h"
#include "gpio_model.h"
#include "board_gpio.h"
#include "pci_model.h"
#include "i2c_model.h"
#include "fpga_xilinx.h"

enum
{
    GPIO_IC_FPGA_XILINX,
    GPIO_IC_NUM,
};

typedef struct
{
    const char *const *pin_name;
    U8_T	pin_no;
}pin_desc_t;

typedef struct
{
	const char *ic_name;
	pin_desc_t *pin_desc;
}gpio_config_t;

static const char *const pin_name[BOARD_GPIO_NUM]=
{
    [BOARD_GPIO_RESET_7619]="PIN_RESETN_7619",
    //[BOARD_GPIO_EDID_WP]="PIN_EDID_WP",
    //[BOARD_GPIO_EDID_W1]="PIN_EDID_W1",
    //[BOARD_GPIO_EDID_W2]="PIN_EDID_W2",
    //[BOARD_GPIO_HPD_RX_CTL]="PIN_HPD_RX_CTL",
};

static pin_desc_t fpga_xilinx_pin_desc[]=
{
    {
	.pin_name=&pin_name[BOARD_GPIO_RESET_7619],
	.pin_no=FPGA_XILINX_GPIO_3, 
    },
    {
        NULL,
    },
};

static gpio_config_t gpio_config[GPIO_IC_NUM]=
{
	{
		.ic_name=FPGA_XILINX_GPIODRV_NAME,
		.pin_desc=fpga_xilinx_pin_desc,
	},
};

int board_gpio_init(cxt_mgr_handle_t cxt_mgr)
{
	gpio_model_handle_t gpio_mgr;
	int i;

        gpio_mgr=cxt_manager_get_context(cxt_mgr,GPIO_CXT_ID,0);
	if(gpio_mgr)
	{
		for(i=0;i<GPIO_IC_NUM;i++)
		{
			pin_desc_t *pin_desc;

			pin_desc=gpio_config[i].pin_desc;
			while(pin_desc->pin_name)
			{
				gpio_model_register_pin(gpio_mgr,gpio_config[i].ic_name,*pin_desc->pin_name,pin_desc->pin_no);
				pin_desc++;
			}
        }
	}

	return 0;
}

void board_set_gpio(cxt_mgr_handle_t cxt_mgr,board_gpio_e no,gpio_model_pin_value_t value)
{
    gpio_model_handle_t gpio_mgr;
    
    do
    {
        gpio_model_pin_handle_t pin_handle;
        if(!cxt_mgr)
            break;
        gpio_mgr=cxt_manager_get_context(cxt_mgr,GPIO_CXT_ID,0);
        if(!gpio_mgr)
            break;
        if(no< 0 || no >BOARD_GPIO_NUM)
            break;
        pin_handle=gpio_model_request_pin_handle(gpio_mgr,pin_name[no]);
        if(!pin_handle)
            break;
        gpio_model_pin_set(pin_handle,value);
        gpio_model_pin_release(pin_handle);
          
    }while(0);
    
}

gpio_model_pin_value_t board_get_gpio(cxt_mgr_handle_t cxt_mgr,board_gpio_e no)
{
    gpio_model_handle_t gpio_mgr;
    gpio_model_pin_value_t value=GPIOIC_VALUE_UNKNOWN;
    
    do
    {
        gpio_model_pin_handle_t pin_handle;
        if(!cxt_mgr)
            break;
        gpio_mgr=cxt_manager_get_context(cxt_mgr,GPIO_CXT_ID,0);
        if(!gpio_mgr)
            break;
        if(no< 0 || no >BOARD_GPIO_NUM)
            break;
        pin_handle=gpio_model_request_pin_handle(gpio_mgr,pin_name[no]);
        if(!pin_handle)
            break;
        value=gpio_model_pin_get(pin_handle);
        gpio_model_pin_release(pin_handle);
    }while(0);
    
    return value;

}

