 /*
 *Driver for the card pcie bridge.

 *Copyright (C) 2019 AVerMedia TECHNOLOGIES, Inc.

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
#include <linux/kernel.h>
#include "cxt_mgr.h"
#include "i2c_model.h"
#include "board.h"
#include "board_i2c.h"
#include "gpio_model.h"
#include "adv7619.h"
#include "sha204.h"
#include "debug.h"
#include "board_gpio.h"
#include "sys.h"


static const char *i2c_bus_name[BOARD_I2C_BUS_NUM]=
{
    [I2C_BUS_COM]="I2C_BUS_COM", //reserved one I2C
};


static adv7619_cfg_t adv7619_cfg=
{
	.out_ctrl={
	.out_fmt=ADV7619_OUT_FORMAT_SDR_ITU656_24_MODE0,
	.out_colorspace=ADV7619_OUT_YUV709,  
	//.out_colorspace=ADV7619_OUT_DEFAULT,   
    },
};

static i2c_dev_info_t cx311h_i2c_bus1_devices[]=
{
    {
        .name=ADV7619_DRVNAME,
		.addr=(0x98>>1),
		.context=&adv7619_cfg, //color space
    },
    {
        NULL,
    } 
};

static i2c_dev_info_t *board_i2c_bus_devices[SUPPORT_BOARD_NUM][BOARD_I2C_BUS_NUM]=
{
  [CN311H][I2C_BUS_COM]= &cx311h_i2c_bus1_devices[0], //reserved one I2C
  
};

const char *board_get_i2c_bus_name(int no)
{
    if(no < 0 || no >= BOARD_I2C_BUS_NUM)
           return NULL;
    return i2c_bus_name[no];
}

int board_i2c_init(cxt_mgr_handle_t cxt_mgr,unsigned long dev_id)
{
	i2c_model_handle_t i2c_mgr=cxt_manager_get_context(cxt_mgr,I2C_CXT_ID,0);
	int err=0;
	int ret=0;

	if(i2c_mgr)
	{
		int i;
		i2c_dev_info_t *dev_info;
            
		board_set_gpio(cxt_mgr,BOARD_GPIO_RESET_7619,GPIOIC_VALUE_LOW);
		sys_msleep(100);
		board_set_gpio(cxt_mgr,BOARD_GPIO_RESET_7619,GPIOIC_VALUE_HIGH);
		sys_msleep(100);     
            
        for(i=0;i<BOARD_I2C_BUS_NUM;i++)
        {
            if(board_i2c_bus_devices[dev_id][i])
            {    
                dev_info=&board_i2c_bus_devices[dev_id][i][0];
                while((err==0) && (dev_info->name!=NULL))
                {
                    if((ret=i2c_model_bus_attach_device(i2c_mgr,i2c_bus_name[i],dev_info->name,dev_info->addr,dev_info->context))==0)
                    {	
                        dev_info++;
                        printk("board_i2c_init ok\n");
                    }
                    else
                    {
                        err=-1;
                        debug_msg("error in attach %s %d\n",dev_info->name,ret);//SHA204 
                        break;
                    }
                }
                if(err)
                {
                    printk("board_i2c_init err\n");
                    break;
                }
            }    
        }    
    }
    return err;
}

