 /*
 *Driver for the debugging.

 *Copyright (C) 2019 AVerMedia TECHNOLOGIES, Inc.

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 

#ifndef DEBUG_H
#define DEBUG_H

#ifdef __cplusplus
extern "C" {
#endif

#define DEBUG 1    

int mesg(const char *fmt,...);
#define mesg_info(fmt, ...) \
        mesg(KERN_INFO fmt, ##__VA_ARGS__)
#define mesg_emerg(fmt, ...) \
        mesg(KERN_EMERG fmt, ##__VA_ARGS__)
#define mesg_alert(fmt, ...) \
        mesg(KERN_ALERT fmt, ##__VA_ARGS__)
#define mesg_crit(fmt, ...) \
        mesg(KERN_CRIT fmt, ##__VA_ARGS__)
#define mesg_err(fmt, ...) \
        mesg(KERN_ERR fmt, ##__VA_ARGS__)
#define mesg_warning(fmt, ...) \
        mesg(KERN_WARNING fmt, ##__VA_ARGS__)
#define mesg_notice(fmt, ...) \
        mesg(KERN_NOTICE fmt, ##__VA_ARGS__)
#if DEBUG
#define mesg_debug(fmt, ...) \
        debug_msg(KERN_DEBUG fmt, ##__VA_ARGS__)

#else
#define mesg_debug(fmt, ...)
#endif

typedef enum
{
    DDEBUG_FLAG_NONE=0,
    DDEBUG_FLAG_SHOW=(0x1<<0),
    DDEBUG_FLAG_INC_FUNC=(0x1<<1),        
    DDEBUG_FLAG_INC_FILE=(0x1<<2),
    DDEBUG_FLAG_INC_LINENO=(0x1<<3),
    DDEBUG_FLAG_DEFAULT=DDEBUG_FLAG_NONE,        
}ddebug_flags_e;

 typedef 
union
 {
    struct 
    {
         const char *function;
         const char *filename;
         unsigned int lineno;
         ddebug_flags_e flags;
    };
    U8_T data[32];   
 }ddebug_t __attribute__((aligned(16)));

#define DEFINE_DDEBUG_METADATA(name)                \
        static ddebug_t   __attribute__((aligned(16))) __attribute__((section("ddebug"))) \
        name = {          \
             .function = __func__,                           \
             .filename = __FILE__,                           \
             .lineno = __LINE__,                             \
             .flags =  DDEBUG_FLAG_DEFAULT,               \
        }
#if 1
#define debug_msg(fmt,...)                  \
    do                                      \
    {                                       \
        DEFINE_DDEBUG_METADATA(descriptor); \
        if(descriptor.flags & DDEBUG_FLAG_SHOW)                \
        {                                                       \
            if(descriptor.flags ^ DDEBUG_FLAG_SHOW)                                            \
            {                                                   \
                if(descriptor.flags & DDEBUG_FLAG_INC_FILE)                \
                    mesg("FILE %s",descriptor.filename);                           \
                if(descriptor.flags & DDEBUG_FLAG_INC_LINENO)                \
                    mesg("LINE %d",descriptor.lineno);                           \
                if(descriptor.flags & DDEBUG_FLAG_INC_FUNC)                \
                    mesg("FUNC %s",descriptor.function);                           \
                mesg(":",descriptor.function);                           \
            }                                                   \
            mesg(fmt, ##__VA_ARGS__);       \
        }                                   \
    }while(0)
#else
#define debug_msg(fmt,...)    mesg(fmt, ##__VA_ARGS__)        
#endif

void init_debug(handle_t trace_handle);
void uninit_debug(void);

#ifdef __cplusplus
}
#endif

#endif /* DEBUG_H */

